// This file is a part of the TTRM Program;
// copyright (C) 2001-2002, Alexandros Droseltis;
// it is distributed under the terms of the
// GNU General Public Licence v.2 (read the file LICENCE)
#include"export.h"
#include<string.h>
#include<stdlib.h>

Export::Export(QWidget *parent=0) : QWidget(parent){
     initialize();

//--------------------------------------------------
// BEGINING(COLUMNS - BUTTONGROUP)
//--------------------------------------------------
    columns=new QButtonGroup("Columns", this);
    columns->setGeometry(10, 10, 175, 150);

    prime=new QCheckBox("Prime", columns);
    prime->setGeometry(10, 20, 160, 25);
    prime->setChecked(true);
    retrograde=new QCheckBox("Retrograde", columns);
    retrograde->setGeometry(10, 20+30, 160, 25);
    retrograde->setChecked(true);
    inversion=new QCheckBox("Inversion", columns);
    inversion->setGeometry(10, 20+2*30, 160, 25);
    inversion->setChecked(true);
    r_inversion=new QCheckBox("Retrograde Inversion", columns);
    r_inversion->setGeometry(10, 20+3*30, 160, 25);
    r_inversion->setChecked(true);
//--------------------------------------------------
// END(COLUMNS - BUTTONGROUP)
//--------------------------------------------------


//--------------------------------------------------
// BEGINING(OTHER OPTIONS - QBUTTONGROUP)
//--------------------------------------------------
    others=new QButtonGroup("Other Options", this);
    others->setGeometry(200, 10, 180, 180);

    int bet=20;
    note_numbers_box=new QCheckBox("Don't print numbers\nunder the notes",
				   others);
    note_numbers_box->setGeometry(10, bet, 160, 30);

    bet+=35;
    end_box=new QCheckBox(
	"Don't print the command\n\\end in the .tex file", others);
    end_box->setGeometry(10, bet, 160, 30);

    bet+=35;
    s_accid_box=new QCheckBox("Print e/b sharp, f/c flat", others);
    s_accid_box->setGeometry(10, bet, 160, 30);

    bet+=35;
    d_accid_box=new QCheckBox("Print double accidentals", others);
    d_accid_box->setGeometry(10, bet, 160, 30);
//--------------------------------------------------
// END(OTHER OPTIONS - QBUTTONGROUP)
//--------------------------------------------------

    int vr=200;
    output=new QButtonGroup("Verticals' Options", this);
    output->setGeometry(10, vr, 370, 60);

    verticals=new QCheckBox("Stravinskian Verticals", output);
    verticals->setGeometry(10, 20, 160, 25);
    forms=new QComboBox(false, output);
    forms->setGeometry(180, 20, 100, 20);
    forms->insertItem("Prime");
    forms->insertItem("Retrograde");
    forms->insertItem("Inversion");
    forms->insertItem("R. Inversion");
    forms->setEnabled(false);
    v_trans=new QSpinBox(0, 11, 1, output);
    v_trans->setGeometry(300, 20, 50, 20);
    v_trans->setEnabled(false);


    connect(verticals, SIGNAL(clicked()), this, SLOT(vert_enable()));
    connect(verticals, SIGNAL(clicked()), this, SIGNAL(ve_clicked()));
    connect(verticals, SIGNAL(stateChanged(int)), this, SIGNAL(ve_clicked()));

    int seira5=440;
    int col1_s5, col2_s5;
    int w_col1_s5=130;
    col1_s5=10;
    col2_s5=col1_s5+w_col1_s5;

    int col3_s5=320;
    set_all();
}

void Export::set_options(void){
    opt[0]=1-end_box->isChecked();
    opt[1]=1-note_numbers_box->isChecked();
    opt[2]=1-s_accid_box->isChecked();
    opt[3]=1-d_accid_box->isChecked();
    opt[4]=1-prime->isChecked(); // make_P
    opt[5]=1-retrograde->isChecked(); // make_R
    opt[6]=1-inversion->isChecked(); // make_I
    opt[7]=1-r_inversion->isChecked(); // make_RI
}

void Export::initialize(void){
    for(int i=0; i<24; i++) opt[i]=1;
    vert_active=0;
    strcpy(head, "");
    strcpy(line_row, "");
    strcpy(title, "");
}

bool Export::get_v(){
    return verticals->isChecked();
}

 void Export::vert_enable(void){
     if(read_mult(qline_row)==12){
	 forms->setEnabled(verticals->isChecked());
	 v_trans->setEnabled(verticals->isChecked());
     }
     else{
	 verticals->setChecked(false);
	 QMessageBox *no_vert=new QMessageBox(this);
	 no_vert->setCaption("ttrm: verticals");
	 no_vert->setText("The row is not a twelve-tone one!");
	 no_vert->setIcon(QMessageBox::Warning);
	 no_vert->show();
     }
}

bool* Export::get_opt(){
   set_options();
   return opt;
}

int Export::get_opt(int i){
   return opt[i];
}

void Export::set_row(QString row){
    qline_row=row;
    strcpy(line_row, row.latin1());
    if(read_mult(row)!=12){
	verticals->setChecked(false);
	forms->setEnabled(false);
	v_trans->setEnabled(false);
    }
}

void Export::set_title(QString qtitle){
    strcpy(title, qtitle.latin1());
}

char* Export::get_row(void){
    return line_row;
}

char* Export::get_title(void){
    return title;
}

void Export::set_form(void){
    form=forms->currentItem();
}

int Export::get_form(void){
    form=forms->currentItem();
    return form;
}

void Export::set_trans(void){
     vert_trans=v_trans->value();
}

int Export::get_trans(void){
    vert_trans=v_trans->value();
    return vert_trans;
}

void Export::set_all(int tmp=0){
     set_options();
     set_form();
     set_trans();
}

int Export::read_mult(QString a_row){
    QStringList *qsep_notes=new QStringList();
    *qsep_notes=QStringList::split(" ", a_row);
    int i=0;
    while((*qsep_notes)[i]!=NULL){
	strcpy(sep_notes[i], (*qsep_notes)[i]);
	i++;
    }
    return i;
}
