// This file is a part of the TTRM Program;
// copyright (C) 2001-2002, Alexandros Droseltis;
// it is distributed under the terms of the
// GNU General Public Licence v.2 (read the file LICENCE)
#include"table.h"
#include<iomanip.h>
#include<string.h>
#include<qstring.h>

Table::Table(){
  texfile=new ofstream;
  num_file=new ofstream;
  ger_file=new ofstream;
}

Table::~Table(){
  //  delete texfile;
}

void Table::set_table(bool opt[], Row myRow){
  set_options(opt);
  setForms(myRow);
  filter();
}

Matrix* Table::get_matrices(void){
    return Forms;
}

void Table::set_filename(const char *name){
  strcpy(filename, name);
//  strcat(filename, "_rows.tex"); // the filename whit suffix
  strcat(filename, ".tex"); // the filename whit suffix

  strcpy(num_filename, name);
//  strcat(num_filename, "_num.txt");
  strcat(num_filename, ".txt");

  strcpy(ger_filename, name);
//  strcat(ger_filename, "_ger.txt");
  strcat(ger_filename, ".txt");
}

void Table::set_options(bool opt[]){
  end=opt[0];
  numbers=opt[1];
  no_es=opt[2];
  no_double=opt[3];
  make_P=opt[4];
  make_R=opt[5];
  make_I=opt[6];
  make_RI=opt[7];
  square=opt[8];
  arithm=opt[9];
}
void Table::set_title(const char *titel){
  strcpy(title, titel);
}

void Table::setForms(Row Seira){
  int i;
  for(i=0; i<4; i++){
    Forms[i].generate(Seira);
  }
  Forms[1].retrograde();
  Forms[2].invert();
  Forms[3].invert();
  Forms[3].retrograde();
}

Matrix Table::getForms(int position){
  return Forms[position];
}

void Table::filter1(void){
  for(int i=0; i<4; i++) Forms[i].filter1();
}

void Table::filter2(void){
  for(int i=0; i<4; i++) Forms[i].filter2();
}

void Table::filter3(void){
  for(int i=0; i<4; i++) Forms[i].filter3();
}

void Table::ambitus(void){
  for(int i=0; i<4; i++) Forms[i].ambitus();
}

void Table::filter(void){
  for(int i=0; i<4; i++){
    if(no_es) Forms[i].filter1();
    if(no_double) Forms[i].filter2();
    Forms[i].filter3();
    Forms[i].ambitus();
  }
}


// The function "accid_filt" gets rid of the unessecary accidantals.
int Table::accid_filt(Row Seira, int pos){
  int j;

  // IF THE ACCIDENTAL OF THE NOTE IS NOT A NATURAL, RETURN 1 (LET IT
  // BE PRINTED).
  if(Seira.getNotes(pos).getAccid()!=0) return 1;

  // IF THE ACCIDENTAL OF THE NOTE IS A NATURAL...
  else{
    // if the note is the first one of the row, return 0
    // (do not let the natural be printed);
      if(pos==0) return (0);
      // if the note is not the first one...
      else{
	// ... start checking the accidentals of the previous notes.
	  for(j=pos; j>0; j--){
	    // If a note is found, that has the same last name
	    // (without accidental), stop the process of checking.
	    if(Seira.getNotes(pos).getHead()==Seira.getNotes(j-1).getHead()){
		if(Seira.getNotes(j-1).getAccid()!=0) return(1);
		if(Seira.getNotes(j-1).getAccid()==0) return(0);
	    }
	  }
	  // If the counter (j) has reached the value of the index
	  // number of the note that is being checked, (i.e. there
	  // is not a previous note whith the same last name),
	  // return 0 (do not let the natural be printed); If not,
	  // return 1
	  if(j==0) return (0);
      }
  }
}

// PRINTERS

void Table::print_row(int slice_pos, int row_pos){
  int i;
  int plithos=getForms(0).get_itsSeira(0).getPlithos();

  texfile->open(filename, ios::app);
  for(i=0; i<plithos; i++){
    if(numbers){
      if(i+1<10) *texfile << "\\numbone{" << i+1 << "}";
      else *texfile << "\\numbtwo{" << i+1 << "}";
    }
    *texfile << "\\nq{";
    if(accid_filt(getForms(slice_pos).get_itsSeira(11-row_pos), i)==1)
      *texfile << num2accid(getForms(slice_pos).get_itsSeira(11-row_pos).getNotes(i).getAccid());
    *texfile <<
      num2notename(getForms(slice_pos).get_itsSeira(11-row_pos).getNotes(i).getHead()) <<
      "}%%\n";
  }
  if(row_pos!=11) *texfile << "&%%\n";
  texfile->close();
}

void Table::print_slice(int slice_pos){
  int j;

  texfile->open(filename, ios::app);
  *texfile << "\\notes%%" <<  endl;
  texfile->close();

  for(j=0; j<11; j++) print_row(slice_pos, j);

  texfile->open(filename, ios::app);
  *texfile << "\\Uptext{\\BIgtype ";

  switch(slice_pos){
  case 0: *texfile << "P"; break;
  case 1: *texfile << "R"; break;
  case 2: *texfile << "I"; break;
  default: *texfile << "RI";
  }
   *texfile << "}%%" << endl;
   texfile->close();

   print_row(slice_pos, 11);
   texfile->open(filename, ios::app);
   *texfile << "\\en\n";
   texfile->close();

}

void Table::print_score(void){
  int summe;

  summe=make_P+make_R+make_I+make_RI;

  if(make_P==0 || summe==4) print_slice(0);
  if(make_P+make_R==0 || make_P+make_I==0 || make_P+make_RI==0 || summe==4)
    print_barline();

  if(make_R==0 || summe==4) print_slice(1);
  if(make_R+make_I==0 || make_R+make_RI==0 || summe==4)
    print_barline();

  if(make_I==0 || summe==4) print_slice(2);
  if(make_I+make_RI==0 || summe==4)
    print_barline();

  if(make_RI==0 || summe==4) print_slice(3);

  texfile->open(filename, ios::app);
  *texfile << "\\endextract\n";
  if(end==1) *texfile << "\\end";
  texfile->close();
}

void Table::print_settings(){
  texfile->open(filename);

  *texfile <<
    "\\input musixtex\n"
    "\\input musixmad\n\n"
    "\\def\\sp{\\off{0.333\\noteskip}}\n\n"
    "\\def\\numbone#1{\\zcharnote{-7}{\\smalltype #1}}\n"
    "\\def\\numbtwo#1{\\zcharnote{-7}{\\loffset{0.5}{\\smalltype #1}}}\n"
    "\\instrumentnumber{12}\n";

  for(int i=0; i<=12; i++)
    *texfile << "\\setname{" << i << "}{" << 12-i << "}\n";

  *texfile <<
    "\\interinstrument = 20pt\n"
    "\\nobarnumbers\n"
    "\\nopagenumbers\n"
    "\\nostartrule\n"
    "\\parindent 10pt\n"
    "\\smallmusicsize\n"
    "\\smallaccid\n"
    "\\hsize=193mm"

    "\\hoffset -0.5in\n"
    "\\centerline{\\BIgtype " << title << "}\n"
    "\\vskip 30pt\n"

    "\n\\startextract\n";

  texfile->close();
}

void Table::print(void){
    print_settings();
    print_score();
}

void Table::tetragono_ger(void){
  int i, j, head;
  char g_note[6];
  int plithos=getForms(0).get_itsSeira(0).getPlithos();
  int accid_p, head_p, accid_i, head_i;

  ger_file->open(ger_filename);

  for(i=0; i<plithos; i++){
      accid_i=getForms(2).get_itsSeira(0).getNotes(i).getAccid();
      head_i=getForms(2).get_itsSeira(0).getNotes(i).getHead();
      head=chromatic_small(accid_i, head_i);
      for(j=0; j<plithos; j++){
	  accid_p=getForms(0).get_itsSeira(head).getNotes(j).getAccid();
	  head_p=getForms(0).get_itsSeira(head).getNotes(j).getHead();
	  german_names(accid_p, head_p, g_note);
	  *ger_file << g_note << "\t";
      }
      *ger_file << endl;
  }
      ger_file->close();
}

void Table::tetragono_num(void){
  int i, j, head;
  char g_note[6];
  int plithos=getForms(0).get_itsSeira(0).getPlithos();
  int accid_p, head_p, accid_i, head_i;

  num_file->open(num_filename);
  for(i=0; i<plithos; i++){
      accid_i=getForms(2).get_itsSeira(0).getNotes(i).getAccid();
      head_i=getForms(2).get_itsSeira(0).getNotes(i).getHead();

      head=chromatic_small(accid_i, head_i);
      for(j=0; j<plithos; j++){
	  accid_p=getForms(0).get_itsSeira(head).getNotes(j).getAccid();
	  head_p=getForms(0).get_itsSeira(head).getNotes(j).getHead();
	  *num_file << setw(2) << chromatic_small(accid_p, head_p) << "\t";
      }
      *num_file << endl;
  }
  num_file->close();
}

const char* Table::tetragono_ger_str(void){
  int i, j, head;
  char g_note[6];
  int plithos=getForms(0).get_itsSeira(0).getPlithos();
  int accid_p, head_p, accid_i, head_i;
  const char* tetragono;
  QString *square_str=new QString();

  for(i=0; i<plithos; i++){
      accid_i=getForms(2).get_itsSeira(0).getNotes(i).getAccid();
      head_i=getForms(2).get_itsSeira(0).getNotes(i).getHead();
      head=chromatic_small(accid_i, head_i);
      for(j=0; j<plithos; j++){
	  accid_p=getForms(0).get_itsSeira(head).getNotes(j).getAccid();
	  head_p=getForms(0).get_itsSeira(head).getNotes(j).getHead();
	  german_names(accid_p, head_p, g_note);
	  *square_str+=QString(g_note);
	  *square_str+="\t";
      }
      *square_str+="\n";
  }
  return square_str->latin1();
}

const char* Table::tetragono_num_str(void){
  int i, j, head;
  char g_note[6];
  int plithos=getForms(0).get_itsSeira(0).getPlithos();
  int accid_p, head_p, accid_i, head_i;
  QString *square_str, *temp;
  square_str=new QString();
  temp=new QString();


  for(i=0; i<plithos; i++){
      accid_i=getForms(2).get_itsSeira(0).getNotes(i).getAccid();
      head_i=getForms(2).get_itsSeira(0).getNotes(i).getHead();

      head=chromatic_small(accid_i, head_i);
      for(j=0; j<plithos; j++){
	  accid_p=getForms(0).get_itsSeira(head).getNotes(j).getAccid();
	  head_p=getForms(0).get_itsSeira(head).getNotes(j).getHead();
	  temp->setNum(chromatic_small(accid_p, head_p));
	  *square_str+=*temp;
	  *square_str+="\t";
      }
      *square_str+="\n";
  }
  return square_str->latin1();
}

int Table::chromatic_small(int x, int y){
   return(chromatic_big(x, y)%12);
}

int Table::chromatic_big(int x, int y){
  if(y%7<3) return(2*(y%7)+x+12*(y/7));
  else return(2*(y%7)+x-1+12*(y/7));
}

// CONVERTORS

char Table::num2accid(int x){
  char accid[]={'<', '_', '=', '^', '>'};
  return(accid[x+2]);
}

char Table::num2notename(int x){
  char heads[]={
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
    'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'};
  return(heads[x-5]);
}

// The following function changes numbers into the german note names.
void Table::german_names(int x, int y, char g_note[]){
  int note;

  note=y%7;
  switch(note){
  case 0: g_note[0]='c'; break;
  case 1: g_note[0]='d'; break;
  case 2: g_note[0]='e'; break;
  case 3: g_note[0]='f'; break;
  case 4: g_note[0]='g'; break;
  case 5: g_note[0]='a'; break;
  default: g_note[0]='h';
  }

  /********** Accidental translator **********/
  /* If the accidental is a flat, */
  if(x==-1){
    /* ... and the note is not a H, an E or an A... */
    if(note!=2 && note!=5 && note!=6){
      g_note[1]='e';
      g_note[2]='s';
      g_note[3]='\0';
    }
    /* If the note is an E or a A... */
    else if(note==2 || note==5){
      g_note[1]='s';
      g_note[2]='\0';
    }
    /* If the note is a H... */
    else{
      g_note[0]='b';
      g_note[1]='\0';
    }
  }
  else if(x==1){
    g_note[1]='i';
    g_note[2]='s';
    g_note[3]='\0';
  }
  else if(x==-2){
    /* ... and the note is not a H, an E or an A... */
    if(note!=2 && note!=5 && note!=6){
      g_note[1]='e';
      g_note[2]='s';
      g_note[3]='e';
      g_note[4]='s';
      g_note[5]='\0';
    }
    /* If the note is an E or a A... */
    else if(note==2 || note==5){
      g_note[1]='s';
      g_note[2]='e';
      g_note[3]='s';
      g_note[4]='\0';
    }
    /* If the note is a H... */
    else{
      g_note[0]='b';
      g_note[1]='e';
      g_note[2]='s';
      g_note[3]='\0';
    }
  }
  else if(x==2){
    g_note[1]='i';
    g_note[2]='s';
    g_note[3]='i';
    g_note[4]='s';
    g_note[5]='\0';
  }
  else if(x==0) g_note[1]='\0';
}

void Table::print_barline(void){
  texfile->open(filename, ios::app);
  *texfile << "\\bar" << endl;
  texfile->close();
}
